@@ -6,9 +6,9 @@ Created on Fri Nov 20 20:22:44 2020
 """
 
 # Python通用工具库，春秋迭代，yondersky@126.com，2020-11-20
-# 更新日期：2022-02-13
+# 更新日期：2022-02-23
 
-import collections, functools, itertools, math, numbers, time
+import collections, functools, itertools, math, numbers, orderedset, time
 import datetime as dt
 import numpy as np
 import pandas as pd
@@ -1407,6 +1407,35 @@ def lexsort(data, ascending = True, na_option = 'last', method = 'first'):
     else:
         return LexsortList(data,ascending,na_option,method)
 
+# 5.3 集合取补
+
+# 5.3.1 下标取补
+
+# 2022-02-23
+def posdiff(size, pos):
+    '''
+    下标取补。
+    【示例】
+    >>> posdiff(5,[1,3])
+    [0, 2, 4]
+    '''
+    pos = set(pos)
+    return [i for i in range(size) if not i in pos]
+
+# 5.3.2 集合取补
+
+# 2022-02-03
+def setdiff(a, b):
+    '''
+    集合取补。
+    【示例】
+    >>> setdiff(list('abcde'),list('ceg'))
+    ['a', 'b', 'd']
+    '''
+    a = orderedset.OrderedSet(a)
+    b = set(b)
+    return [i for i in a if not i in b]
+
 # 6. 逻辑扩展
 
 # 2021-12-02
